/*
 *  Copyright (C) 2009  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <iksemel.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

iks *
createiq (char *type, char *to, char *qnam, char *xmlns, iks **query)
{
  static int id = 0;
  char sid[32];;
  iks *iq;
  snprintf (sid, 32, "ps%d", id++);
  iq = iks_new ("iq");
  iks_insert_attrib (iq, "type", type);
  iks_insert_attrib (iq, "to", to);
  iks_insert_attrib (iq, "id", sid);
  *query = iks_insert (iq, qnam);
  iks_insert_attrib (*query, "xmlns", xmlns);
  return iq;
}

void
catnode (iksparser *parser, char *node)
{
  iks *iq;
  iks *query;
  iq = createiq ("get", "pubsub.jabber-br.org", "query",
                 "http://jabber.org/protocol/disco#info", &query);
  if (node != NULL)
    iks_insert_attrib (query, "node", node);
  iks_send (parser, iq);
  iks_delete (iq);
}

void
listnode (iksparser *parser, char *node)
{
  iks *iq;
  iks *query;
  iq = createiq ("get", "pubsub.jabber-br.org", "query",
                 "http://jabber.org/protocol/disco#items", &query);
  if (node != NULL)
    iks_insert_attrib (query, "node", node);
  iks_send (parser, iq);
  iks_delete (iq);
}

void
createnode (iksparser *parser, char *node)
{
  iks *iq;
  iks *query;
  iq = createiq ("set", "pubsub.jabber-br.org", "pubsub",
                 "http://jabber.org/protocol/pubsub", &query);
  iks_insert_attrib (iks_insert (query, "create"), "node", node);
  iks_send (parser, iq);
  iks_delete (iq);
}

void
getnode (iksparser *parser, char *node)
{
  iks *iq;
  iks *query;
  iq = createiq ("get", "pubsub.jabber-br.org", "pubsub",
                 "http://jabber.org/protocol/pubsub", &query);
  iks_insert_attrib (iks_insert (query, "items"), "node", node);
  iks_send (parser, iq);
  iks_delete (iq);
}

void
vcard (iksparser *parser)
{
  iks *iq;
  iks *query;
  iq = createiq ("get", "pubsub.jabber-br.org", "vCard", "vcard-temp", &query);
  iks_send (parser, iq);
  iks_delete (iq);
}

iks *
createmood (char *line)
{
  iks *mood;
  mood = iks_new ("mood");
  iks_insert_attrib (mood, "xmlns", "http://jabber.org/protocol/mood");
  iks_insert (mood, line);
  return mood;
}

void
pushmood (iksparser *parser, char *node, char *line)
{
  iks *iq;
  iks *query;
  iks *publish;
  iks *item;
  iks *mood;
  iq = createiq ("set", "pubsub.jabber-br.org", "pubsub",
                 "http://jabber.org/protocol/pubsub", &query);
  publish = iks_insert (query, "publish");
  iks_insert_attrib (publish, "node", node);
  item = iks_insert (publish, "item");
  mood = createmood (line);
  iks_insert_node (item, mood);
  iks_send (parser, iq);
  iks_delete (iq);
}

void
process_mood (iksparser *parser, char *cmdline)
{
  char *cmd;
  char *orig_cmdline;
  orig_cmdline = cmdline = strdup (cmdline);
  cmd = strsep (&cmdline, " ");
  if (!strcmp (cmd, "ls"))
    listnode (parser, cmdline);
  else if (!strcmp (cmd, "cat"))
    catnode (parser, cmdline);
  else if (!strcmp (cmd, "vcard"))
    vcard (parser);
  else if (!strcmp (cmd, "create"))
    createnode (parser, cmdline);
  else if (!strcmp (cmd, "get"))
    getnode (parser, cmdline);
  else if (!strcmp (cmd, "push"))
    {
      char *node;
      node = strsep (&cmdline, " ");
      pushmood (parser, node, cmdline);
    }
  free (orig_cmdline);
}

int
xmpp_session_hook (iksparser *parser, iks *node)
{
  iks *iq;
  iq = iks_new ("iq");
  iks_insert_attrib (iq, "type", "set");
  iks_insert_attrib (iq, "id", "session1");
  iks_insert_attrib (iks_insert (iq, "session"),
                     "xmlns", "urn:ietf:params:xml:ns:xmpp-session");
  iks_send (parser, iq);
  iks_delete (iq);
  return 0;
}

int
xmpp_initial_presence_hook (iksparser *parser, iks *node)
{
  iks *pres;
  pres = iks_make_pres (IKS_SHOW_AVAILABLE, "Microblogging here!");
  iks_send (parser, pres);
  iks_delete (pres);
  return 0;
}

int
xmpp_id_hook (iksparser *parser, iks *node, char *id)
{
  if (!iks_strcmp (id, "bind1"))
    {
      xmpp_session_hook (parser, node);
      return 0;
    }
  else if (!iks_strcmp (id, "session1"))
    {
      xmpp_initial_presence_hook (parser, node);
      return 0;
    }
  else if (!iks_strncmp (id, "ps", 2))
    {
      printf ("%s\n", iks_string (iks_stack (node), node));
    }
  return 1;
}

int
xmpp_ns_hook (iksparser *parser, iks *node, char *ns)
{
  return 1;
}

int
xmpp_iq_error (iksparser *parser, iks *node)
{
  iks *enode;
  char *to;
  char *from;
  if (!iks_strcmp (iks_find_attrib (node, "type"), "error"))
    return 1;
  to = iks_find_attrib (node, "to");
  from = iks_find_attrib (node, "from");
  if (to)
    iks_insert_attrib (node, "from", to);
  else
    iks_insert_attrib (node, "from", NULL);
  if (from)
    iks_insert_attrib (node, "to", from);
  else
    iks_insert_attrib (node, "to", NULL);
  iks_insert_attrib (node, "type", "error");
  enode = iks_insert (node, "error");
  iks_insert_attrib (enode, "type", "cancel");
  iks_insert_attrib (iks_insert (enode, "feature-not-implemented"),
                     "xmlns", "urn:ietf:params:xml:ns:xmpp-stanzas");
  iks_send (parser, node);
  return 0;
}

int
xmpp_tls_hook (iksparser *parser, iks *node)
{
  iks_start_tls (parser);
  return 0;
}

int
xmpp_sasl_hook (iksparser *parser, iks* node)
{
  iks_start_sasl (parser, IKS_SASL_DIGEST_MD5, "pubsub", "pubsub");
  return 0;
}

int
xmpp_bind_hook (iksparser *parser, iks *node)
{
  iks *iq;
  iq = iks_new ("iq");
  iks_insert_attrib (iq, "type", "set");
  iks_insert_attrib (iq, "id", "bind1");
  iks_insert_attrib (iks_insert (iq, "bind"),
                     "xmlns", "urn:ietf:params:xml:ns:xmpp-bind");
  iks_send (parser, iq);
  iks_delete (iq);
  return 0;
}

int
xmpp_features_hook (iksparser *parser, iks *node)
{
  iks *feat;
  char *ns;
  for (feat = iks_child (node); feat != NULL; feat = iks_next (feat))
    {
      ns = iks_find_attrib (feat, "xmlns");
      if (!iks_strcmp (ns, "urn:ietf:params:xml:ns:xmpp-tls"))
        {
          xmpp_tls_hook (parser, node);
          return 0;
        }
      else if (!iks_strcmp (ns, "urn:ietf:params:xml:ns:xmpp-sasl"))
        {
          xmpp_sasl_hook (parser, node);
          return 0;
        }
      else if (!iks_strcmp (ns, "urn:ietf:params:xml:ns:xmpp-bind"))
        {
          xmpp_bind_hook (parser, node);
          return 0;
        }
    }
  return 1;
}

int
xmpp_other_hook (iksparser *parser, iks *node, char *ns)
{
  if (!iks_strcmp (ns, "urn:ietf:params:xml:ns:xmpp-sasl"))
    {
      if (!iks_strcmp (iks_name (node), "success"))
        iks_send_header (parser, "jabber-br.org");
      else if (!iks_strcmp (iks_name (node), "failure"))
        printf ("failture to authenticate: %s\n",
                iks_string (iks_stack (node), node));
      return 0;
    }
  return 1;
}

const char * jid = "cascardo@jabber-br.org";

static int
hook (void *data, int type, iks *node)
{
  char *name;
  char *id;
  char *ns;
  char *iqns;
  iksparser *parser;
  parser = *(iksparser **) data;
  name = iks_name (node);
  id = iks_find_attrib (node, "id");
  ns = iks_find_attrib (node, "xmlns"); 
  iqns = iks_find_attrib (iks_child (node), "xmlns"); 
  if (!iks_strcmp (name, "message"))
    {
      char *from;
      from = iks_find_attrib (node, "from");
      if (!iks_strncmp (from, jid, iks_strlen (jid)))
        {
          char *body = iks_find_cdata (node, "body");
          if (body != NULL)
            process_mood (parser, body);
        }
      else
        {
          printf ("%s is not authorized\n", from);
        }
      return IKS_OK;
    }
  else if (!iks_strcmp (name, "presence"))
    {
      char *from;
      from = iks_find_attrib (node, "from");
      printf ("I sense a disturbance in the force: %s!\n", from);
    }
  else if (!iks_strcmp (name, "iq"))
    {
      if (xmpp_id_hook (parser, node, id) == 0)
        return IKS_OK;
      if (xmpp_ns_hook (parser, node, iqns) == 0)
        return IKS_OK;
      xmpp_iq_error (parser, node);
    }
  else if (!iks_strcmp (name, "stream:features"))
    {
      if (xmpp_features_hook (parser, node) == 0)
        return IKS_OK;
    }
  else if (!iks_strcmp (name, "stream:error"))
    {
      printf ("streamerror: %s\n", iks_string (iks_stack (node), node));
    }
  else
    {
      if (xmpp_other_hook (parser, node, ns) == 0)
        return IKS_OK;
      printf ("No no: %s\n", name);
    }
  return IKS_OK;
}

int
main (int argc, char **argv)
{
  iksparser *parser;
  parser = iks_stream_new ("jabber:client", &parser, hook);
  iks_connect_tcp (parser, "jabber-br.org", 5222);
  while (1)
    iks_recv (parser, -1);
  return 0;
}
